#############################################################################
# calculator.tcl Copyright 2004 r0lZ
# Simple Tcl/Tk calculator.
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

namespace eval ::calc {
namespace export calculator

# Calculator.
# arguments: w window name.
proc calculator {w} {
	if [winfo exists $w] {
		wm deiconify $w
		return
	}
	toplevel $w ;#-class Dialog
	wm withdraw $w
	wm transient $w .
	wm title $w "Tcl Calculator, Hex & String Converter"

	frame $w.f1
	pack  $w.f1 -fill x -expand 1
	entry $w.f1.calc -textvariable ::calc::calc -width 42
	pack  $w.f1.calc -fill x -side left -expand 1
	entry $w.f1.ascii -textvariable ::calc::ascii -width 4 -validate key \
		-validatecommand {return [expr [string length %P] <= 4]} \
		-font $::config(fixed_width_font)
	pack  $w.f1.ascii -side left
	bind  $w.f1.calc <KeyPress-Return> ::calc::do_calculation
	bind  $w.f1.calc <Escape> {set ::calc::calc ""; set ::calc::result ""}
	bind  $w.f1.ascii <KeyPress-Return> ::calc::do_ascii_conversion
	bind  $w.f1.ascii <Escape> {set ::calc::ascii ""; set ::calc::result ""}
	entry $w.result -textvariable ::calc::result -state readonly
	pack  $w.result -fill x
	update
	wm resizable $w true false
	wm minsize $w [winfo reqwidth $w] [winfo reqheight $w]
	#if {$::tcl_platform(platform) == "windows"} {
	#	catch {wm attributes $w -toolwindow 1}
	#}
	wm deiconify $w
	focus $w.f1.calc
}
proc do_calculation {} {
	if {[catch {set r [expr $::calc::calc]} errmsg] != 0} {
		set ::calc::result [string range $errmsg [expr [string first ": " $errmsg]+2] end]
	} else {
		if [string is integer $r] {
			set ascii ""
			set num $r
			while {$num > 0} {
				set c [expr $num % 256]
				set num [expr $num / 256]
				set ascii "[format {%c} $c]$ascii"
			}
			if {[string length $ascii] <= 8} {
				set ::calc::ascii $ascii
				if {[string length $ascii] > 0} {
						set ::calc::result "= $r = 0x[format %0X $r] = \"$ascii\""
					} else {
						set ::calc::result "= $r = 0x[format %0X $r]"
					}
			} else {
				set ::calc::ascii ""
				set ::calc::result "= $r = 0x[format %0X $r]"
			}
		} else {
			set ::calc::ascii ""
			set ::calc::result "= $r"
		}
	}
}
proc do_ascii_conversion {} {
	set numchars [binary scan $::calc::ascii cccc a(1) a(2) a(3) a(4)]
	set r 0
	for {set d 1} {$d <= $numchars} {incr d} {
		set r [expr ($r * 256) + $a($d)]
	}
	set ::calc::result "= $r = 0x[format %0X $r] = \"$::calc::ascii\""
	set ::calc::calc $r
}

}
