#############################################################################
# preview.tcl Copyright 2004 r0lZ
# preview functions (for Windows): frontend for PgcEditPreview.exe
#
# This file is part of PgcEdit.
#
# PgcEdit is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PgcEdit is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#############################################################################

namespace eval ::preview {

proc preview_current_pgc_2 {} {
	preview2 [lindex $::editor_cmds_from 0] [lindex $::editor_cmds_from 1] 0 0 true
}

## preview a PGC, or a specific cell in the PGC
## If cell argument == 0, then preview all cells of the PGC, else preview the specified cell
proc preview2 {type index cell autoquit in_background} {
	if {$::tcl_platform(platform) != "windows"} return
	set indexslist [split $index ,]
	if {$type == "pgc"} {
		set vts [lindex $indexslist 0]
		if {$vts == 0} {bell; return}
		set lu  0
		set pgc [lindex $indexslist 1]
		set numcells $::pgcs($vts,$pgc,numcells)
		set cellpbtable $::pgcs($vts,$pgc,cellpbtable)
		set label [::main::pgc_label $::config(fixed_width_font) $type $vts $pgc]
	} else {
		set vts [lindex $indexslist 0]
		set lu  [lindex $indexslist 1]
		set pgc [lindex $indexslist 2]
		set numcells $::menus($vts,$lu,$pgc,numcells)
		set cellpbtable $::menus($vts,$lu,$pgc,cellpbtable)
		set label [::main::pgc_label $::config(fixed_width_font) $type $vts $lu $pgc]
	}
	if {$numcells == 0} {bell; return}
	set newlabel [string map {"  " " "} $label]
	while {$newlabel != $label} {
		set label $newlabel
		set newlabel [string map {"  " " "} $label]
	}
	set label [string map {" ," ","} $label]

	if {$vts == 0} {
		set vobfile  [file nativename [file join $::dvddir "VIDEO_TS.VOB"]]
		set dar [expr ([get_int ::raw_sectors(0,VMGM_MAT) 0x100 2] & 0x0C00)]
	} else {
		if {$type == "menu"} {
			set vobfile [file nativename [file join $::dvddir "VTS_[format %02d $vts]_0.VOB"]]
			set dar [expr ([get_int ::raw_sectors($vts,VTSI_MAT) 0x100 2] & 0x0C00)]
		} else {
			set vobfile [file nativename [file join $::dvddir "VTS_[format %02d $vts]_1.VOB"]]
			set dar [expr ([get_int ::raw_sectors($vts,VTSI_MAT) 0x200 2] & 0x0C00)]
		}
	}
	if {! [file exists $vobfile]} {bell; return}
	if {$dar == [expr 0x0C00]} {set dar 16} else {set dar 4}

	set lbastxt ""
	if {$cell == 0} {
		for {set c 1} {$c <= $numcells} {incr c} {
			set cellidx [expr ($c - 1) * 24]
			set anglemode [expr [get_int cellpbtable $cellidx 1] & 0xF0]
			set lbastxt "$lbastxt[get_int cellpbtable [expr $cellidx + 8] 4] "
			switch $anglemode {
				80 - 144 {	;# first and middle angle: pass entry_VOBU_sector and first_ILVU_end
					set lbastxt "$lbastxt[get_int cellpbtable [expr $cellidx +12] 4] "
				}
				default {	;# normal and last_angle of block: pass entry_VOBU_sector and last_VOBU_end
					set lbastxt "$lbastxt[get_int cellpbtable [expr $cellidx +20] 4] "
				}
			}
		}
		if {$numcells == 1} {set cell 1}
	} else {
		set numcells 1
		set cellidx [expr ($cell - 1) * 24]
		set lbastxt "$lbastxt[get_int cellpbtable [expr $cellidx + 8] 4] "
		set lbastxt "$lbastxt[get_int cellpbtable [expr $cellidx +20] 4] "
	}
	if {$lbastxt == ""} {bell; return}

	if $in_background {set delay 10000} else {set delay 3000}

	set txt "\[PgcEditPreview\]\n"
	set txt "${txt}VOBFileName = $vobfile\n"
	set txt "${txt}NumCells = $numcells\n"
	set txt "${txt}CellStartEnds = $lbastxt\n"
	set txt "${txt}DAR = $dar\n"
	set txt "${txt}Scale = [expr round(100 / $::config(preview_screen_divisor))]\n"
	set txt "${txt}PGCTitle = $label\n"
	set txt "${txt}DefaultFileName = [file nativename [file join $::config(config_directory) "preview.cfg.ini"]]\n"
	set txt "${txt}CellNumber = $cell\n"
	set txt "${txt}StartPlay = $::config(preview_autoplay)\n"	;# TODO: don't autoplay when still frames...
	set txt "${txt}AutoLocate = $::config(preview_autolocate)\n"
	set txt "${txt}AutoQuit = $autoquit\n"
	set txt "${txt}SleepOnSingleFrame = $delay\n"
	set txt "${txt}ThisID = [pid]\n"

	if {$::config(debug) > 0} {
		puts $txt
	}

	set progname [file nativename [file join $::config(bindir) "PgcEditPreview.exe"]]
	set tempfile [file nativename [file join $::config(tempdir) "preview.args.ini"]]
	catch {file delete $tempfile}
	if {[saveFile $tempfile auto $txt] != 0} {bell; return}
	if {$in_background} {
		catch {exec -- $progname "$tempfile" &}
	} else {
		catch {exec -- $progname "$tempfile"}
	}
}

proc close_preview {} {
	if {$::tcl_platform(platform) != "windows"} return
	set txt "\[PgcEditPreview\]\n"
	set txt "${txt}ThisID = [pid]\n"
	set txt "${txt}KillSelf = 1\n"

	set progname [file nativename [file join $::config(bindir) "PgcEditPreview.exe"]]
	set tempfile [file nativename [file join $::config(tempdir) "preview.args.ini"]]
	catch {file delete $tempfile}
	catch {saveFile $tempfile auto $txt}
	catch {exec -- $progname "$tempfile" &}
}

}	;# end namespace ::preview
